/*
	Copyright (C) 2025  snoutie
	Authors: snoutie (copyright@achtarmig.org)
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU Affero General Public License as published
	by the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Affero General Public License for more details.

	You should have received a copy of the GNU Affero General Public License
	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

package table

import (
	"api-cds-search/cmd/database"
	_ "embed"
)

type Keyword struct {
	CDSViewTechnicalName TechnicalName
	Keyword              string
}

//go:embed sql/query_all_keywords.sql
var query_all_keywords string

func GetAllKeywords() (*[]Keyword, error) {
	rows, err := database.DB.Query(query_all_keywords)
	if err != nil {
		return nil, err
	}
	var keywords []Keyword
	for rows.Next() {
		var keyword Keyword
		err := rows.Scan(&keyword.CDSViewTechnicalName, &keyword.Keyword)
		if err != nil {
			return nil, err
		}
		keywords = append(keywords, keyword)
	}
	return &keywords, nil
}

//go:embed sql/insert_or_replace_keyword.sql
var insert_or_replace_keyword string

func InsertOrReplaceKeyword(CDSViewTechnicalName string, Keywords string) error {
	_, err := database.DB.Exec(insert_or_replace_keyword, CDSViewTechnicalName, Keywords)
	if err != nil {
		return err
	}
	return nil
}
